(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
    typeof define === 'function' && define.amd ? define(factory) :
    (global = global || self, global.blobs = factory());
}(this, function () { 'use strict';

    var loopAccess = function (arr) { return function (i) {
        return arr[((i % arr.length) + arr.length) % arr.length];
    }; };
    var rad = function (deg) {
        return (deg / 360) * 2 * Math.PI;
    };
    var deg = function (rad) {
        return (((rad / Math.PI) * 1) / 2) * 360;
    };
    var distance = function (p1, p2) {
        return Math.sqrt(Math.pow((p1.x - p2.x), 2) + Math.pow((p1.y - p2.y), 2));
    };
    var angle = function (p1, p2) {
        return deg(Math.atan2(p2.y - p1.y, p2.x - p1.x));
    };
    var interpolate = function (point, opt) {
        var handles = point.handles || { angle: 0, out: 0, in: 0 };
        handles.angle = Math.PI + rad(handles.angle);
        return {
            x: point.x,
            y: opt.height - point.y,
            handles: handles,
        };
    };
    var smooth = function (points, opt) {
        if (points.length === 2)
            return points;
        var out = [];
        for (var i = 0; i < points.length; i++) {
            var point = loopAccess(points)(i);
            var before = loopAccess(points)(i - 1);
            var after = loopAccess(points)(i + 1);
            out.push({
                x: point.x,
                y: point.y,
                handles: {
                    angle: angle(before, after),
                    in: opt.strength * (1 / 2) * distance(point, before),
                    out: opt.strength * (1 / 2) * distance(point, after),
                },
            });
        }
        return out;
    };
    var rand = function (seed) {
        var xfnv1a = function (str) {
            var h = 2166136261 >>> 0;
            for (var i = 0; i < str.length; i++) {
                h = Math.imul(h ^ str.charCodeAt(i), 16777619);
            }
            return function () {
                h += h << 13;
                h ^= h >>> 7;
                h += h << 3;
                h ^= h >>> 17;
                return (h += h << 5) >>> 0;
            };
        };
        var sfc32 = function (a, b, c, d) { return function () {
            a >>>= 0;
            b >>>= 0;
            c >>>= 0;
            d >>>= 0;
            var t = (a + b) | 0;
            a = b ^ (b >>> 9);
            b = (c + (c << 3)) | 0;
            c = (c << 21) | (c >>> 11);
            d = (d + 1) | 0;
            t = (t + d) | 0;
            c = (c + t) | 0;
            return (t >>> 0) / 4294967296;
        }; };
        var seedGenerator = xfnv1a(seed);
        return sfc32(seedGenerator(), seedGenerator(), seedGenerator(), seedGenerator());
    };

    var render = function (p, opt) {
        var points = p.map(function (point) { return interpolate(point, opt); });
        var handles = [];
        for (var i = 0; i < points.length; i++) {
            var _a = loopAccess(points)(i), x = _a.x, y = _a.y, hands = _a.handles;
            var next = loopAccess(points)(i + 1);
            var nextHandles = next.handles;
            if (hands === undefined) {
                handles.push({ x1: x, y1: y, x2: next.x, y2: next.y });
                continue;
            }
            handles.push({
                x1: x - Math.cos(hands.angle) * hands.out,
                y1: y + Math.sin(hands.angle) * hands.out,
                x2: next.x + Math.cos(nextHandles.angle) * nextHandles.in,
                y2: next.y - Math.sin(nextHandles.angle) * nextHandles.in,
            });
        }
        var path = "";
        for (var i = 0; i <= points.length; i++) {
            var point = loopAccess(points)(i);
            var hands = loopAccess(handles)(i - 1);
            if (i === 0) {
                path += "M" + point.x + "," + point.y;
                continue;
            }
            path += "C" + hands.x1 + "," + hands.y1 + "," + hands.x2 + "," + hands.y2 + "," + point.x + "," + point.y;
        }
        var guides = "";
        if (opt.guides) {
            var color = opt.stroke || "black";
            var size = opt.strokeWidth || 1;
            if (opt.boundingBox) {
                guides += "\n                <rect x=\"0\" y=\"0\" width=\"" + opt.width + "\" height=\"" + opt.height + "\" fill=\"none\"\n                    stroke=\"" + color + "\" stroke-width=\"" + 2 * size + "\" stroke-dasharray=\"" + 2 * size + "\" />";
            }
            for (var i = 0; i < points.length; i++) {
                var _b = loopAccess(points)(i), x = _b.x, y = _b.y;
                var hands = loopAccess(handles)(i);
                var nextPoint = loopAccess(points)(i + 1);
                guides += "\n                <line x1=\"" + x + "\" y1=\"" + y + "\" x2=\"" + hands.x1 + "\" y2=\"" + hands.y1 + "\"\n                    stroke-width=\"" + size + "\" stroke=\"" + color + "\" />\n                <line x1=\"" + nextPoint.x + "\" y1=\"" + nextPoint.y + "\" x2=\"" + hands.x2 + "\" y2=\"" + hands.y2 + "\"\n                    stroke-width=\"" + size + "\" stroke=\"" + color + "\" stroke-dasharray=\"" + 2 * size + "\" />\n                <circle cx=\"" + hands.x1 + "\" cy=\"" + hands.y1 + "\" r=\"" + size + "\"\n                    fill=\"" + color + "\" />\n                <circle cx=\"" + hands.x2 + "\" cy=\"" + hands.y2 + "\" r=\"" + size + "\"\n                    fill=\"" + color + "\" />\n                <circle cx=\"" + x + "\" cy=\"" + y + "\" r=\"" + 2 * size + "\" fill=\"" + color + "\" />";
            }
        }
        var stroke = opt.stroke || (opt.guides ? "black" : "none");
        var strokeWidth = opt.strokeWidth || (opt.guides ? 1 : 0);
        return ("\n        <svg\n            width=\"" + opt.width + "\"\n            height=\"" + opt.height + "\"\n            viewBox=\"0 0 " + opt.width + " " + opt.height + "\"\n            xmlns=\"http://www.w3.org/2000/svg\"\n        >\n            <g transform=\"" + (opt.transform || "") + "\">\n                <path\n                    stroke=\"" + stroke + "\"\n                    stroke-width=\"" + strokeWidth + "\"\n                    fill=\"" + (opt.fill || "none") + "\"\n                    d=\"" + path + "\"\n                />\n                " + guides + "\n            </g>\n        </svg>\n    ").replace(/\s+/g, " ");
    };

    var blobs = function (opt) {
        var rgen = rand(opt.seed || String(Date.now()));
        if (!opt.stroke && !opt.color) {
            throw new Error("no color or stroke specified");
        }
        if (opt.complexity <= 0 || opt.complexity > 1) {
            throw new Error("complexity out of range ]0,1]");
        }
        if (opt.contrast < 0 || opt.contrast > 1) {
            throw new Error("contrast out of range [0,1]");
        }
        var count = 3 + Math.floor(14 * opt.complexity);
        var angle$$1 = 360 / count;
        var radius = opt.size / Math.E;
        var points = [];
        for (var i = 0; i < count; i++) {
            var rand_1 = 1 - 0.8 * opt.contrast * rgen();
            points.push({
                x: Math.sin(rad(i * angle$$1)) * radius * rand_1 + opt.size / 2,
                y: Math.cos(rad(i * angle$$1)) * radius * rand_1 + opt.size / 2,
            });
        }
        var smoothed = smooth(points, {
            closed: true,
            strength: ((4 / 3) * Math.tan(rad(angle$$1 / 4))) / Math.sin(rad(angle$$1 / 2)),
        });
        return render(smoothed, {
            closed: true,
            width: opt.size,
            height: opt.size,
            fill: opt.color,
            transform: "rotate(" + rgen() * angle$$1 + "," + opt.size / 2 + "," + opt.size / 2 + ")",
            stroke: opt.stroke && opt.stroke.color,
            strokeWidth: opt.stroke && opt.stroke.width,
            guides: opt.guides,
        });
    };

    return blobs;

}));
